package business;

import rmi.*;
import network.ListenerCommunicator;
import access.DatabaseConstants;
import access.Database;
import view.MainFrame;

import access.RegisteredProgramsDB;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Stack;



public class   ListenerManager implements DatabaseConstants
{

    //------------------------------------------------------------
    static private Hashtable htRecursiveEvents = new Hashtable();
    static private Stack stSoundProperties=new Stack();
    static private Stack stActivityPlayer = new Stack();
    static private Hashtable htRecursiveActivitiesList = new Hashtable();
    static private Hashtable htRecursiveActivityPlayerList = new Hashtable();
    //------------------------------------------------------------
    static private RegisterManager objRM;
    static private SoundSynthesizer objSS;
    static private NewActivityPlayer objAP;

    static boolean bDatabaseCreated=false;
    private ListenerCommunicator objLC;
    static private Recorder objPlayingRecordedProgram= null;
    static private Hashtable htRecordedPrograms = new Hashtable();
    static private Vector vProgramsBuffer = new Vector();
    static private Vector vEventsBuffer= new Vector();
    static private boolean bStatus = false;


    /**
     *
     */
    public ListenerManager()
    {
        initializeDatabase();

        objRM = new RegisterManager();

        objSS = new SoundSynthesizer();

        try
        {
            objLC = new ListenerCommunicator();
        }
        catch(Exception re)
        {
            re.printStackTrace();
            System.out.println("Remote Exception while initialising Listener ");
        }

        if( objLC.initialise() == false)
        {
            //get data from GUI;
            //update data from gui and call initialise.
        }
        initialiseRecorders();
    }



    public  void enablePlayingStatus()
    {
        //MainFrame.enablePlayingButtonStatus();
    }

    public void mute(boolean bStatus)
    {
        if(objPlayingRecordedProgram != null && objPlayingRecordedProgram.retrieveRecordedProgram()!=null)
        {
            //objPlayingRecordedProgram.mute(bStatus);
        }
    }

    public boolean removeRecordedProgram(String sProgramName)
    {
       RegisteredProgramsDB objDB= new RegisteredProgramsDB();
       return objDB.removeRecordedProgram(sProgramName);
    }


    public void updateProgramMapping(String sProgramName, Vector vEventNames, Vector vEventSoundProperties)
    {
       objRM.updateProgramMapping(sProgramName, vEventNames, vEventSoundProperties);
    }

    public void changeEventSoundMapping(String sProgramName, String sEventName, Vector vSoundProperties)
    {
        objRM.changeEventSoundMapping(sProgramName, sEventName, vSoundProperties);
    }

    public Vector getSoundProperties(String sProgramName, String sEventName)
    {
        return objRM.getSoundProperties(sProgramName, sEventName);
    }

    public Vector getEventSoundMappings(String sProgramName)
    {
        return objRM.getEventSoundMappings(sProgramName);
    }

    public boolean setNetworkInfo(String sConfServerIPAddress, String sConfServerPortNo, String sListenerIPAddress, String sListenerPortNo)
    {
        return objLC.setNetworkInfo(sConfServerIPAddress, sConfServerPortNo, sListenerIPAddress, sListenerPortNo);
    }

    public boolean setConfigServerInfo(String sConfServerIPAddress, String sConfServerPortNo)
    {
        return objLC.setConfigServerInfo(sConfServerIPAddress, sConfServerPortNo);
    }

    public boolean setListenerInfo(String sListenerIPAddress, String sListenerPortNo)
    {
        return objLC.setListenerInfo(sListenerIPAddress, sListenerPortNo);
    }

    public String[] getConfigServerInfo()
    {
        return objLC.getConfigServerInfo();
    }

    public String[] getListenerInfo()
    {
        return objLC.getListenerInfo();
    }


    public void initialiseRecorders()
    {
        try
        {
            Vector vVector=retrieveRecordedProgramsList();
            if(vVector!=null)
            {
                for(int iLoop=0; iLoop < vVector.size(); iLoop++)
                {
                    String sProgramName = (String) vVector.elementAt(iLoop);
                    htRecordedPrograms.put(sProgramName,new Recorder(sProgramName));
                }
            }
        }
        catch(Exception e)
        {
            //e.printStackTrace();
            System.out.println("RECOREDER UNABLE TO FIND DB");
        }
    }


    public boolean registerListener()
    {
        /*if(bDatabaseCreated==false)
        {
            initializeDatabase();
        }*/
        return objLC.registerListener();
    }

    private boolean writeRecordedProgramsList()
    {

        Enumeration objENUM = htRecordedPrograms.keys();
        Vector vProgramsList = new Vector();
         System.out.println("Program Recorded");
        while(objENUM.hasMoreElements()==true)
        {
            vProgramsList.add(objENUM.nextElement());
        }

        Database objDB = new Database();

        return objDB.writeObject(RECORDED_PROGRAMS_LIST+RECORDED_PROGRAMS_LIST_FILE_EXTENSION, vProgramsList);
    }

    private Vector retrieveRecordedProgramsList()
    {
        Database objDB = new Database();
        try
        {
            return (Vector)objDB.readObject(RECORDED_PROGRAMS_LIST+RECORDED_PROGRAMS_LIST_FILE_EXTENSION);
        }
        catch(Exception e)
        {
            return null;
        }
    }

    public Vector[] loginListener(String sIPAddress, String sPort)
    {
        /*if(makeConfServerConnection()==false)
            return null;*/
        Vector vProgramList = objLC.loginListener(sIPAddress, sPort);
        if(vProgramList == null)
        {
            System.out.println("Login Failed");
            return null;
        }
        Vector vArray[]= new Vector[2];
        vArray[0]=objRM.getProgramNames();
        /*
        vArray[0] contains Registered Programs
        vArray[1] contains UnRegistered Programs
        */
        vArray[1]= new Vector();
        for(int iLoop=0; iLoop < vProgramList.size(); iLoop++)
        {
            String sProgramName = (String)vProgramList.get(iLoop);
            if(vArray[0].contains(sProgramName) == false)
            {
                vArray[1].add(sProgramName);
            }
        }
        return vArray;
    }
    public boolean logoutListener(String sIPAddress, String sPort)
    {
        return objLC.logoutListener(sIPAddress, sPort);
    }


    public void initializeDatabase()
    {
        bDatabaseCreated=true;
        Database objDB = new Database();
        objDB.createFolders();
    }
    /**
     * to be implemented
     * @return
     */
    public boolean unRegisterListener()
    {
       /* if(makeConfServerConnection()==false)
            return false;*/

        boolean bUnregisterStatus = false;
        bUnregisterStatus = objLC.unRegisterListener();
        if(bUnregisterStatus == true)
        {
            Database objDB= new Database();
            objDB.deleteFolders();
            return true;
        }
        else
        {
            return false;
        }

    }

    /**
     * to be implemented
     * @param sProgramName
     * @return
     */
    public boolean registerProgram(String sProgramName)
    {
        System.out.println("Register Program Called for Program : " + sProgramName);
        Hashtable htMappings= objLC.registerProgram(sProgramName);
        if(htMappings==null)
            return false;
        else
            return objRM.registerProgram(sProgramName,htMappings);

    }

    /**
     * to be implemented
     * @param sProgramName
     * @return
     */
    public boolean unRegisterProgram(String sProgramName)
    {
        System.out.println("UnRegister Program Called for Program : " + sProgramName);
        if(objLC.unRegisterProgram(sProgramName) == true)
            return objRM.unRegisterProgram(sProgramName);
        return false;
    }

    public boolean playRecordedSound(String sProgramName)
    {
        objPlayingRecordedProgram = new Recorder(sProgramName);
        if(objPlayingRecordedProgram.retrieveRecordedProgram()!=null)
        {

            System.out.println("Inside playRecordedSound - Recorder Object"+ objPlayingRecordedProgram);
            System.out.println("Program Name : " + sProgramName);
            objPlayingRecordedProgram.playRecordedSound();
            return true;
        }
        return false;

        /*Database objDB= new Database();
        try
        {
            vRecordedVector=(Vector[])objDB.readObject(RECORDED_PROGRAM+sProgramName+RECORDED_PROGRAM_FILE_EXTENSION);

            if(vRecordedVector == null)
            {
                //System.out.println("Recorded Vector : NULL in playRecordedSound");
                return false;
            }
            bPlayRecordedProgram=true;
            sRecordedProgramName=sProgramName;
            playEvents();
        }
        catch(Exception e)
        {
            System.out.println("Exception while Playing Recored Events");
            e.printStackTrace();
            return false;
        }
        return true;*/
    }

    public void startRecording(String sProgramName)
    {
        htRecordedPrograms.put(sProgramName,new Recorder(sProgramName));
        this.writeRecordedProgramsList();
    }

    public void stopRecording(String sProgramName)
    {
        htRecordedPrograms.remove(sProgramName);
        this.writeRecordedProgramsList();
    }

    public void stopPlayingRecordedProgram(String sProgramName)
    {

        if(objPlayingRecordedProgram != null && objPlayingRecordedProgram.retrieveRecordedProgram()!=null)
        {
            System.out.println("Inside Listener Manager, stop Playing Sound - Recorder Object"+ objPlayingRecordedProgram);
            System.out.println("Program Name : " + sProgramName);
            objPlayingRecordedProgram.stopRecordedSound();
        }
        objPlayingRecordedProgram=null;
    }



    public Vector getRecordedPrograms()
    {
        Enumeration objEnum =  htRecordedPrograms.keys();

        Vector vPrograms = null;

        if(objEnum != null)
        {
            vPrograms= new Vector();
            while(objEnum.hasMoreElements() == true)
            {
                vPrograms.add(objEnum.nextElement());
            }
        }
        return vPrograms;
    }

    public void stopSound(String sProgramName)
    {
        Recorder objRecorder = new Recorder(sProgramName);
        if(objRecorder.retrieveRecordedProgram()!=null)
        {
            objRecorder.playRecordedSound();

        }
    }

    public static Vector getInstrumentNames()
    {
        return objSS.getInstrumentNames();
    }
    public Vector getEventsForProgram(String sProgramName)
    {
       return objRM.getEventSoundHandlers(sProgramName);
    }

    public static  void sendNotify(String sProgName, EventInfo objEInfo)
    {
        System.out.println("Send Notify called");
        vEventsBuffer.add(objEInfo);
        vProgramsBuffer.add(sProgName);

        bStatus = true;
         Recorder objRecorder=null;

         if(htRecordedPrograms.containsKey(sProgName)==true)
         {
             System.out.println("------------------------------");
            objRecorder = (Recorder)htRecordedPrograms.get(sProgName);

            if(objEInfo.getEventName().equalsIgnoreCase("START") == true)
            {
                System.out.println("Start Event for Recorder-Timer");
                objRecorder.setStartTime();

            }

            //objRecorder.recordEvent(objEI);

            else if(objEInfo.getEventName().equalsIgnoreCase("END") == true)
            {
                System.out.println("Recorded Program Stored");
                objRecorder.storeRecordedProgram();
                htRecordedPrograms.put(sProgName, new Recorder(sProgName));

            }
            /*else
            {
                //objRecorder.recordEvent(objEI);
            }*/
         }

        //playEvents();

        if(bStatus == true)
        {
            for(int iLoop=0;iLoop< vProgramsBuffer.size(); iLoop++)
            {
                String sProgramName = (String)vProgramsBuffer.remove(0);
                EventInfo objEI = (EventInfo)vEventsBuffer.remove(0);

                Vector vSoundProperties = objRM.getSoundProperties(sProgramName, objEI.getEventName());

                System.out.println("Going to play Event: "+objEI.getEventName());
                if(htRecordedPrograms.containsKey(sProgramName)==true)
                {
                    /*if(objEI.getEventName().indexOf("recursion_method_body_begin")!= -1)
                    {
                        if(htRecursiveEvents.containsKey(objEI.getEventName()))
                        {

                        }else
                        {
                          htRecursiveEvents.put(objEI.getEventName(),new Integer());
                        }
                    }else if(objEI.getEventName().indexOf("recursion_method_body_end") != -1){
                    }
                       */
                    synchronized(objRecorder)
                    {

                    objRecorder = (Recorder)htRecordedPrograms.get(sProgName);
                    if(objEI.isActivity()==false)
                        objRecorder.recordEvent(vSoundProperties, objEI.getData());
                    }
                }

                if(vSoundProperties != null)
                {
                    if(objEI.isActivity() == true)
                    {
                        if(objEI.getActivityType().equalsIgnoreCase("START") == true)
                        {

                            if(stSoundProperties.isEmpty())
                            {
                                stSoundProperties.push(vSoundProperties);
                                objAP=new NewActivityPlayer(objRecorder);
                                stActivityPlayer.push(objAP);
//----------------------------------------------------------------------------------
                                //System.out.println("*******NAME   "+objEI.getEventName());
                                if(htRecursiveActivitiesList.containsKey(objEI.getEventName()))
                                {

                                    int iNote=((Integer)htRecursiveActivitiesList.get(objEI.getEventName())).intValue();
                                    iNote+=1;  //incrementing by 5 notes for clarity
                                    objAP.changeNote(iNote);
                                   // System.out.println("$$$$$$$$$$$ NOTE INCREMENTED  "+iNote);
                                    htRecursiveActivitiesList.put(objEI.getEventName(),new Integer(iNote));
                                }
                                else if(objEI.getEventName().startsWith("atrack_recursion_method"))
                                {
                                    //System.out.println("NEW ATRACH ADDED@@@@@@@@@@@@@@");
                                    htRecursiveActivityPlayerList.put(objEI.getEventName(),objAP);
                                    htRecursiveActivitiesList.put(objEI.getEventName(),new Integer(objAP.getNote()));
                                }
//----------------------------------------------------------------------------------
                                objAP.playActivitiesSound(vSoundProperties);
                                //System.out.println("playing AP==="+objAP);
                            }
                            else
                            {
                                objAP= ((NewActivityPlayer)(stActivityPlayer.peek()));
                                //System.out.println("SUSPEND currently playing AP bcos of FUNCTION CALL = " + objAP);
                                objAP.suspendThread();


                                stSoundProperties.push(vSoundProperties);

                                objAP=new NewActivityPlayer(objRecorder);

//----------------------------------------------------------------------------------
                                if(htRecursiveActivitiesList.containsKey(objEI.getEventName()))
                                {

                                    int iNote=((Integer)htRecursiveActivitiesList.get(objEI.getEventName())).intValue();
                                    iNote++;
                                    objAP.changeNote(iNote);
                                    //System.out.println("$$$$$$$$$$$ NOTE INCREMENTED  "+iNote);
                                    htRecursiveActivitiesList.put(objEI.getEventName(),new Integer(iNote));
                                }
                                else if(objEI.getEventName().startsWith("atrack_recursion_method"))
                                {
                                   // System.out.println("NEW ATRACH ADDED@@@@@@@@@@@@@@");
                                    htRecursiveActivityPlayerList.put(objEI.getEventName(),objAP);
                                    htRecursiveActivitiesList.put(objEI.getEventName(),new Integer(objAP.getNote()));
                                }
//----------------------------------------------------------------------------------

                                stActivityPlayer.push(objAP);

                                objAP.playActivitiesSound(vSoundProperties);
                               // System.out.println("playing new AP==="+objAP);
                            }


                        }
                        else if(objEI.getActivityType().equalsIgnoreCase("END") == true)
                        {

                           if(!stActivityPlayer.isEmpty())
                           {
                               objAP=((NewActivityPlayer)(stActivityPlayer.pop()));
                               stSoundProperties.pop();
//--------------------------------------------------------------------------------------

                                if(htRecursiveActivitiesList.containsKey(objEI.getEventName()))
                                {

                                    NewActivityPlayer objTemp=(NewActivityPlayer)htRecursiveActivityPlayerList.get(objEI.getEventName());
                                    if(objTemp.equals(objAP))
                                    {
                                        htRecursiveActivitiesList.remove(objEI.getEventName());
                                       // System.out.println("=-=-=-=-=-=-=-=-=-=-=-=-=-DELETED RECURSION");
                                    }
                                }
//--------------------------------------------------------------------------------------
                              // System.out.println("stopped current AP==="+objAP);
                               objAP.stopActivitiesSound();


                               if(!stActivityPlayer.isEmpty()) //after stopping the current activity, plays the previously executed activity.
                               {
                                   objAP=((NewActivityPlayer)(stActivityPlayer.peek()));
                                    objAP.resumeThread();
                                     // System.out.println("RESUME previous  AP==="+objAP);
                               }
                           }
                        }
                    }
                    else
                    {
                        //objSS.playSound(vSoundProperties, objEI.getData());
                        objSS.playNote(vSoundProperties, objEI.getData());
                    }
                }
                else
                {
                   // System.out.println("Sound Properties is Null For  : " + objEI.getEventName());
                }

            }
        }


    }
}
