package business;

import javax.sound.midi.*;
import java.util.Vector;
import java.util.Hashtable;

public class NewActivityPlayer extends Thread implements MIDIConstants{


    private Synthesizer objSynth = null;
    private ChannelData objChannels[] = null;
    private ChannelData cc=null;

    private Hashtable htInstruments = null;
    private Instrument objInstruments[] = null;

    private final int CHANNEL_NUMBER_ACTIVITY = 0;

    private boolean bExitFlag=false;
    private boolean bSuspend=false;
    private boolean bNoteIncremented=false;

    //for recording activity
    private Recorder objRecorder=null;

    private int iData = NOTE_ON;

    private Vector vSoundProperties=null;

    public NewActivityPlayer(Recorder objRecorder)
    {
        this.objRecorder=objRecorder;
        htInstruments = new Hashtable();

        open();
        mapInstrumentNames();
        cc.channel.controlChange(REVERB, cc.reverb);
        cc.channel.setChannelPressure(cc.pressure);
        cc.channel.setPitchBend(cc.bend);


    }

    /**
     * Stores MidiChannel information.
     */

    class ChannelData {

        MidiChannel channel;
        boolean solo, mono, mute, sustain;
        int velocity, pressure, bend, reverb;
        // int row, col;
        int num;

        public ChannelData(MidiChannel channel, int num) {
            this.channel = channel;
            this.num = num;
            velocity = pressure = reverb = 64;
            bend=8192;
        }
    } // End class ChannelData


    public void changeNote(int iNote)
    {
        bNoteIncremented=true;
        iData=((++iNote)%MAX_NOTES == 0)?NOTE_ON:iNote;

    }

    public int getNote()
    {
        return iData;
    }
    public void open() {
        try {
            if (objSynth == null) {
                if ((objSynth = MidiSystem.getSynthesizer()) == null) {
                    System.out.println("getSynthesizer() failed!");
                    return;
                }
            }
            objSynth.open();

            // objSequencer = MidiSystem.getSequencer();
            //objSequence = new Sequence(Sequence.PPQ, 10);
        } catch (Exception ex) { ex.printStackTrace(); return; }

        Soundbank sb = objSynth.getDefaultSoundbank();
        if (sb != null) {
            objInstruments = objSynth.getDefaultSoundbank().getInstruments();
            //objSynth.loadInstrument(objInstruments[0]);
        }
        MidiChannel midiChannels[] = objSynth.getChannels();
        objChannels = new NewActivityPlayer.ChannelData[midiChannels.length];
        for (int i = 0; i < objChannels.length; i++) {
            objChannels[i] = new NewActivityPlayer.ChannelData(midiChannels[i], i);
        }
        cc = objChannels[CHANNEL_NUMBER_ACTIVITY];
    }


    public void playActivitiesSound(Vector vSoundProperties)
    {
        this.vSoundProperties=vSoundProperties;
        this.start();
    }



    private void mapInstrumentNames()
    {

        for(int iLoop=0; iLoop < MIDI_INSTRUMENTS_LENGTH; iLoop++)
        {
            htInstruments.put(objInstruments[iLoop].getName().trim(),new Integer(iLoop));
            //System.out.println("MAP------INSTR-"+htInstruments.get(objInstruments[iLoop].getName())+"--VAL---"+htInstruments.get(objInstruments[iLoop].getName()));
        }

    }

    public void run()
    {
        String sSoundVariable = (String)vSoundProperties.get(0);
        //int iPitch = ((Integer)vSoundProperties.get(1)).intValue();
        int iNote = ((Integer)vSoundProperties.get(1)).intValue();
        String sTime = (String)vSoundProperties.get(2);
        boolean bStatus = ((Boolean)(vSoundProperties.get(3))).booleanValue();

        //Newly added on 22nd Mar 2004
        if(bNoteIncremented==false)
        {
            this.iData = iNote;
        }else{
            bNoteIncremented=false;
        }

        int iInstrument=0;
        try{
            iInstrument= ((Integer)htInstruments.get(sSoundVariable)).intValue();
        }
        catch(Exception e)
        {
            System.out.println("INSTRUMENT NOT FOUND");
        }

        if(bStatus)
        {
            objSynth.loadInstrument(objInstruments[iInstrument]);
            cc.channel.programChange(iInstrument);
            while(!bExitFlag)
            {
                if(bSuspend)
                {
                }
                else
                {
                    cc.channel.noteOn(iData, cc.velocity);
                    try
                    {
                        Thread.sleep(SOUND_DELAY);
                    }catch(Exception e)
                    {
                    }
                    cc.channel.noteOff(iData, cc.velocity);
                    if(objRecorder != null)
                    {
                        synchronized(objRecorder)
                        {

                            objRecorder.recordEvent(vSoundProperties,new Integer(iData));
                        }
                    }

                }
            }
            bExitFlag=false;
        }
       // cc.channel.allSoundOff();
    }

    public void suspendThread()
    {
        bSuspend=true;
    }

    public void resumeThread(){
        bSuspend=false;
    }
    public void stopActivitiesSound()
    {
        bExitFlag=true;

    }

}